#!/usr/bin/env python3
import argparse
import xml.etree.ElementTree as et
import inflection
import os
import sys
import csv
from zcross import Molecule, Electron

def main():
    parser = argparse.ArgumentParser(description = 'Update ZCross XML fields')
    group_get = parser.add_argument_group('Getters')
    group_set = parser.add_argument_group('Setters')

    xsd      = et.parse('/opt/zcross/share/zcross/zcross.xsd')
    xsd_root = xsd.getroot()

    ns = {'xs':'http://www.w3.org/2001/XMLSchema', 'zc': 'https://zcross.net', '': 'https://zcross.net', }

    tag_map = {}

    def decode_xsd_type(xsd_type):
        
        if xsd_type in ('xs:string'):
            return (str, 'STRING')
        if xsd_type in ('xs:integer','xs:int'):
            return (int, 'INT')
        if xsd_type in ('xs:boolean'):
            return (bool, 'BOOL')
        if xsd_type in ('xs:double', 'xs:decimal'):
            return (float, 'FLOAT')
        if xsd_type in ('xs:dateTime'):
            return (str, 'DATETIME')
        raise ValueError(f'Unable to decode: {xsd_type}')
        
    def decode_zc_type(xsd_root, zc_type):
        
        simple_type_xsd = xsd_root.find("xs:simpleType[@name='{}']".format(zc_type.split(':')[1]), ns)
        if simple_type_xsd is not None:
            restriction_xsd = simple_type_xsd.find("xs:restriction", ns)
            if restriction_xsd is not None:
                
                parser_type, parser_metavar = decode_xsd_type(restriction_xsd.get('base'))
                
                choices = []
                
                for enumaration_xsd in restriction_xsd.findall("xs:enumeration", ns):
                    choices.append(parser_type(enumaration_xsd.get('value')))
                
                return (parser_type, parser_metavar, choices) #TODO
          
        raise ValueError(f'Unable to decode: {zc_type}')


    def parse_xsd_attribute(xsd_root, xsd_attribute, parser, span, parent):
        attr_name = xsd_attribute.get('name')
        attr_type = xsd_attribute.get('type')
        
        parm_name = '-'.join([inflection.dasherize(inflection.underscore(p)) for p in parent + [attr_name]][-span:])
        
        parm_help = 'the {} attribute'.format(inflection.dasherize(inflection.underscore(attr_name)).replace('-',' '))
        
        if (len(parent) > 1):
            parm_help += ' of the {} element'.format(parent[-1])
        
        if attr_type.startswith('zc:'):
            parser_type, parser_metavar, parser_choices = decode_zc_type(xsd_root, attr_type)
            group_set.add_argument('--set-' + parm_name, type=parser_type, metavar=parser_metavar, choices = parser_choices,  help='Set ' + parm_help)
            group_get.add_argument('--get-' + parm_name, help='Get ' + parm_help, action='store_true')
            tag_map[parm_name] = parent + ['@' + attr_name]
        else:
            parser_type, parser_metavar = decode_xsd_type(attr_type)
            group_set.add_argument('--set-' + parm_name, type=parser_type, metavar=parser_metavar,    help='Set ' + parm_help)
            group_get.add_argument('--get-' + parm_name, help='Get ' + parm_help, action='store_true')
            tag_map[parm_name] = parent + ['@' + attr_name]

    def parse_xsd_element(xsd_root, xsd_element, parser, span, parent):
        
        if xsd_element.get('ref') is not None:
            
            if xsd_element.get('ref').startswith('zc:'):
                child_name = xsd_element.get('ref').split(':')[1]
                xsd_element_new =  xsd_root.find("xs:element[@name='{}']".format(child_name), ns)
                parse_xsd(xsd_root, xsd_element_new, parser, span, parent + [child_name])
                
        elif xsd_element.get('name') is not None:
            
            elem_name = xsd_element.get('name')
            elem_type = xsd_element.get('type')
        
            parm_name = '-'.join([inflection.dasherize(inflection.underscore(p)) for p in parent + [elem_name]][-span:])
            
            parm_help = 'the {} element'.format(parm_name.replace('-',' '))
            
           
            if elem_type.startswith('zc:'):
                parser_type, parser_metavar, parser_choices = decode_zc_type(xsd_root, elem_type)
                group_set.add_argument('--set-' + parm_name, type=parser_type, metavar=parser_metavar, choices = parser_choices, help='Set ' + parm_help)
                group_get.add_argument('--get-' + parm_name, help='Get ' + parm_help, action='store_true')
                tag_map[parm_name] = parent + [elem_name]
            else:
                parser_type, parser_metavar = decode_xsd_type(elem_type)
                group_set.add_argument('--set-' + parm_name, type=parser_type, metavar=parser_metavar, help='Set ' + parm_help)
                group_get.add_argument('--get-' + parm_name, help='Get ' + parm_help, action='store_true')
                tag_map[parm_name] =  parent + [elem_name]
            
    def parse_xsd(xsd_root, xsd_element, parser, span=2, parent=[]):

        if xsd_element.get('name') not in ('reactants', 'products'):
            for xsd_attribute in xsd_element.findall("xs:complexType/xs:attribute", ns):
                parse_xsd_attribute(xsd_root, xsd_attribute, parser, span, parent)
                    
            for xsd_subelement in xsd_element.findall("xs:complexType/xs:sequence/xs:element", ns):
                parse_xsd_element(xsd_root, xsd_subelement, parser, span, parent)
                
            for xsd_subelement in xsd_element.findall("xs:complexType/xs:choice/xs:element", ns):
                parse_xsd_element(xsd_root, xsd_subelement, parser, span + 1, parent)
                    
                   
    parser.add_argument('xml')
    parser.add_argument('database_id', nargs='?')
    parser.add_argument('group_id',    nargs='?')
    parser.add_argument('process_id',  nargs='?')

    xsd_zcross = xsd_root.find("xs:element[@name='zcross']", ns)
    parse_xsd(xsd_root, xsd_zcross, parser)

    group_set.add_argument('--set-reactant-1', metavar='SPECIE' )
    group_set.add_argument('--set-reactant-2', metavar='SPECIE' )
    group_set.add_argument('--set-product-1',  metavar='SPECIE' )
    group_set.add_argument('--set-product-2',  metavar='SPECIE' )
    group_set.add_argument('--set-product-3',  metavar='SPECIE' )
    group_set.add_argument('--set-product-4',  metavar='SPECIE' )
    group_set.add_argument('--set-product-5',  metavar='SPECIE' )
    
    group_set.add_argument('--set-database-reference',  nargs=2)

    group_get.add_argument('--get-reactant-1', action='store_true' )
    group_get.add_argument('--get-reactant-2', action='store_true' )
    group_get.add_argument('--get-product-1',  action='store_true' )
    group_get.add_argument('--get-product-2',  action='store_true' )
    group_get.add_argument('--get-product-3',  action='store_true' )
    group_get.add_argument('--get-product-4',  action='store_true' )
    group_get.add_argument('--get-product-5',  action='store_true' )
    
    group_set.add_argument('--get-database-reference', type=int)
    
    parser.add_argument('--load-data', metavar='CSV')
    parser.add_argument('--save-data', metavar='CSV')
    
    parser.add_argument('--unit-energy', default='eV',  metavar='UNIT')
    parser.add_argument('--unit-cross-section',   default='m^2', metavar='UNIT')

    args = parser.parse_args()

    et.register_namespace('','https://zcross.net')
    et.register_namespace('xsi', 'http://www.w3.org/2001/XMLSchema-instance')

    has_get = False
    has_set = False

    for arg, value in vars(args).items():
        if arg.startswith('get_') and value == True:
            has_get = True
        if arg.startswith('set_') and value is not None:
            has_set = True
        
    document = None
    document_root = None

    if os.path.exists(args.xml):
        document      = et.parse(args.xml)
        document_root = document.getroot()
    else:
        if has_get and not has_set:
            print(f'Unable to open file: {args.xml}')
            sys.exit(1)
        else:
            document_root = et.Element('{https://zcross.net}zcross')
            document = et.ElementTree(document_root)

    def generate_xpath(path):

        tokens = []
        
        for p in path:
            if not p.startswith('@'):
                token = 'zc:' + p
                if p == 'database':
                    if args.database_id is not None:
                        token += '[@id=\'{}\']'.format(args.database_id)
                    else:
                        print("ERROR: specity the database id")
                        sys.exit(1)
                        
                if p == 'group':
                    if args.group_id is not None:
                        token += '[@id=\'{}\']'.format(args.group_id)
                    else:
                        print("ERROR: specity the group id")
                        sys.exit(1)
                        
                if p == 'process':
                    if args.process_id is not None:
                        token += '[@id=\'{}\']'.format(args.process_id)
                    else:
                        print("ERROR: specity the process id")
                        sys.exit(1)
                tokens.append(token)
                
        return '/'.join(tokens) if tokens else '.'
        
        
    def create_xpath(document_root, path, database_id, group_id, process_id):
        
        
        for i in range(len(path)):
            if not path[i].startswith('@'):
                subelement =  document_root.find(generate_xpath(path[:i+1]), ns)
                if subelement is None:
                    parelement = document_root.find(generate_xpath(path[:i]), ns) if len(path[:i]) > 0 else document_root
                    newelement = et.SubElement(parelement, '{https://zcross.net}'+path[i])
                    
                    if path[i] == 'database':
                        newelement.set('id', database_id)
                    if path[i] == 'group':
                        newelement.set('id', group_id)
                    if path[i] == 'process':
                        newelement.set('id', process_id)
                        
               

    for arg, value in vars(args).items():
        
        custom_tag_map = {
            'reactant-1': ['database', 'groups', 'group', 'processes', 'process', 'reactants'],
            'reactant-2': ['database', 'groups', 'group', 'processes', 'process', 'reactants'],
            'product-1':  ['database', 'groups', 'group', 'processes', 'process', 'products' ],
            'product-2':  ['database', 'groups', 'group', 'processes', 'process', 'products' ],
            'product-3':  ['database', 'groups', 'group', 'processes', 'process', 'products' ],
            'product-4':  ['database', 'groups', 'group', 'processes', 'process', 'products' ],
            'product-5':  ['database', 'groups', 'group', 'processes', 'process', 'products' ],
        }
        
        
        if arg == 'get_database_reference' and value is not None:
            pass
        elif arg == 'set_database_reference' and value is not None:
            
            create_xpath(document_root, ['database', 'references'], args.database_id, args.group_id, args.process_id)
            
            xpath = generate_xpath(['database', 'references']) 
            element_references =  document_root.find(xpath, ns)
            
            element = element_references.find(generate_xpath(['reference']) + '[@id="{}"]'.format(value[0]), ns)
            
            if element is None:
                element = et.SubElement(element_references, '{https://zcross.net}reference', {'id': str(value[0])})
            
            element.text = value[1]
            
        elif arg.startswith('get_') and arg[4:].replace('_','-') in custom_tag_map  and value == True:
            path  = custom_tag_map[arg[4:].replace('_','-')]
            
            xpath = generate_xpath(path)
            element =  document_root.find(xpath, ns)        
            subelement = element.getchildren()[int(arg[4:].split('_')[-1])-1]
            
            specie = None
            if subelement is not None:
                if subelement.tag == '{https://zcross.net}electron':
                    specie = Electron(subelement)
                elif subelement.tag == '{https://zcross.net}molecule':
                    specie = Molecule(subelement)
                    
            print('{:<25} : {}'.format(arg[4:].replace('_',' '), str(specie)))
                
        elif arg.startswith('set_') and arg[4:].replace('_','-') in custom_tag_map  and value is not None:
            
            path  = custom_tag_map[arg[4:].replace('_','-')]
            
            create_xpath(document_root, path, args.database_id, args.group_id, args.process_id)
            
            xpath = generate_xpath(path)
            element =  document_root.find(xpath, ns)
            
            pos = int(arg[4:].split('_')[-1])-1
              
            while len(element.getchildren()) <= pos:
                et.SubElement(element, '{https://zcross.net}molecule')
                
            subelement = element.getchildren()[pos]
                
            if subelement is not None:
                element.remove(subelement)
                
            if value == 'e':
                element.insert(pos, et.Element('{https://zcross.net}electron'))
            else:
                value = value.strip()
                state=''
                if value.endswith(')'):
                    level = 0
                    for c in reversed(value):
                        if c == '(':
                            level -= 1
                        if level > 0:
                            state = c + state
                        if c == ')':
                            level += 1
                        value = value[:-1]
                        if level == 0:
                           break
                           
                state = state.strip()
                value = value.strip()
                charge = 0
                while value.endswith('^+'):
                    value = value[:-2]
                    charge += 1
                while value.endswith('^-'):
                    value = value[:-2]
                    charge -= 1      
                new_element = et.Element('{https://zcross.net}molecule')
                new_element.text = value

                if charge != 0:
                    new_element.set('charge', str(charge))
                if state != '':
                    new_element.set('state', state)
                
                element.insert(pos, new_element)   
                    
        elif arg.startswith('get_') and value == True:  
            path = tag_map[arg[4:].replace('_','-')]
              
            xpath  = generate_xpath(path)
            xattr  = None if not path[-1].startswith('@') else path[-1][1:]

            element =  document_root.find(xpath, ns)
                   
            if element is not None:
                print('{:<25} : {}'.format(arg[4:].replace('_',' '), (element.text if xattr is None else element.get(xattr)) or '')) 
            else:
                print('{:<25} : '.format(arg[4:].replace('_',' ')))
                
        elif arg.startswith('set_') and value is not None:
            path = tag_map[arg[4:].replace('_','-')]
            
            create_xpath(document_root, path, args.database_id, args.group_id, args.process_id)
                       
            element = document_root.find(generate_xpath(path), ns)
            xattr  = None if not path[-1].startswith('@') else path[-1][1:]
           
            if xattr is None:
               element.text = str(value)
            else:
               element.set(xattr, str(value))
               
        elif arg == 'load_data' and value is not None:
            
            data = []
            with open(value) as csvfile:
                
                dialect = csv.Sniffer().sniff(csvfile.read(1024))
                csvfile.seek(0)
                reader = csv.reader(csvfile, dialect)
                
                for row in reader:
                    try:
                        float(row[0])
                        float(row[1])
                    except ValueError:
                        continue
                    
                    if len(row) < 2:
                        continue
                        
                    data.append((float(row[0]),float(row[1])))
            
            path = ['database', 'groups', 'group', 'processes', 'process']
            create_xpath(document_root, path, args.database_id, args.group_id, args.process_id)
            
            process_element = document_root.find(generate_xpath(path), ns)
            data_x_element = process_element.find(generate_xpath(['data_x']), ns)
            data_y_element = process_element.find(generate_xpath(['data_y']), ns)
            data_z_element = process_element.find(generate_xpath(['data_z']), ns)
            size_element   = process_element.find(generate_xpath(['size']), ns)

            if data_x_element is not None:
                data_x_element = process_element.find(generate_xpath(['data_x']), ns)
                process_element.remove(data_x_element)
            if data_y_element is not None:
                process_element.remove(data_y_element)
            if data_z_element is not None:
                process_element.remove(data_z_element)
            if size_element is not None:
                process_element.remove(size_element)
           
            size_element   = et.SubElement(process_element, '{https://zcross.net}size')            
            data_x_element = et.SubElement(process_element, '{https://zcross.net}data_x')
            data_y_element = et.SubElement(process_element, '{https://zcross.net}data_y')
            #data_z_element = et.SubElement(process_element, '{https://zcross.net}data_z')

            data_x_element.text = ' '.join([str(d[0]) for d in data])
            data_y_element.text = ' '.join([str(d[1]) for d in data])
            size_element.text   = str(len(data))
            
            data_x_element.set('type', 'energy')
            data_x_element.set('unit', args.unit_energy)
            
            data_y_element.set('type', 'cross_section')
            data_y_element.set('unit', args.unit_cross_section)
            
    if has_set or args.load_data is not None:
        document.write(args.xml,
            xml_declaration = True,
            encoding = 'utf-8',
            method = 'xml')

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import zcross
import argparse
import re
import sys
import matplotlib.pyplot as plt
import os
from pathlib import Path
from cycler import cycler
from pint import UnitRegistry



def main():
    ureg = UnitRegistry()

    parser = argparse.ArgumentParser()

    parser.add_argument('-b', '--bullet', default=None, help='Filter by bullet')
    parser.add_argument('-t', '--target', default=None, help='Filter by target')
    parser.add_argument('-c', '--collision', default=None, help='Filter by collision type')
    parser.add_argument('-e', '--exact',     action='store_true', help='Show only exact match')
    parser.add_argument(      '--velocity',  action='store_true', help='Show bullet velocity instead of energy')
    parser.add_argument('-v', '--verbose',   action='store_true', help='Does nothing, for compatibility')
    parser.add_argument('--title',     help='Title of the plot')
    parser.add_argument('--legend',     default='ids', choices=['ids', 'types', 'none'], help='Format of the legend to show')
    

    parser.add_argument('-o', '--output', default=None, help='Save plot to PNG')

    parser.add_argument('table', nargs='*')
    args = parser.parse_args()

    filter_bullet = args.bullet
    filter_target = args.target
    filter_collission = args.collision
    filter_exact = args.exact

    atoms = {}
      
        
    def calculate_mass(bullet):
        
        if len(atoms) == 0:
            import csv  
            
            csvPath = Path(__file__).parent.parent / 'share' / 'zcross'
            
            for filename in ('periodic.csv', 'isotopes.csv'):
                with open(csvPath / filename) as f:
                    
                    reader = csv.reader(f)
                    
                    header = next(reader, None)  # take the header
                    
                    keySymbol = header.index('Symbol')
                    keyMass   = header.index('Atomic Weight')
                    
                    for line in reader:
                        atoms[line[keySymbol]] = float(line[keyMass]) * ureg.u
                   
        if isinstance(bullet, Electron):
            return ureg.electron_mass
        elif isinstance(bullet, Molecule):
            if len(bullet.value) > 2:
                raise ValueError('Unable to calculate the mass for complex moecule: ' + bullet.value)
            return atoms[bullet]
        else:
            raise ValueError('Unable to calculate the mass for the object: ' + str(bullet))
            
        
        
        
        
    unit_energy   = ureg('eV')
    unit_velocity = ureg('cm/us')
    unit_area     = ureg('m^2')


    r = re.compile(r'^([\w\-\*]*)(?:\/([\w\-\*\(\)]+)(?:\/([\d\*\-\,]+))?)?$')

    filters = args.table

    if len(filters) == 0:
        filters.append('*')


    zs = []


    ms = [ r.match(filter) for filter in filters if filter != '']

    if any([m.group(1) == '*' for m in ms]):
        zs = zcross.load_all()
    else:
        for name in [m.group(1) for m in ms]:
            zs.append(zcross.load_by_name(name))
            
    processes = {}
    labels = {}

    for z in zs:
        database = z.database
        for group in database.groups.group:
            for process in group.processes.process:
                
                accept = False
                
                for m in ms:
                    acceptFilter = True
                    acceptFilter &= not (m.group(1) is not None and m.group(1) != '*' and m.group(1).lower() != database.id.lower())
                    acceptFilter &= not (m.group(2) is not None and m.group(2) != '*' and m.group(2).lower() != group.id.lower())
                    
                    if acceptFilter and m.group(3)  is not None and m.group(3) != '*':
                        
                        acceptToken = False
                        for token in m.group(3).split(','):
                            if '-' in token:
                                id_min,id_max = [int(v) if v != '' else None for v in token.split('-')]
                                acceptToken |=  (id_min is None or process.id >= id_min) and (id_max is None or process.id <= id_max)
                            else:
                                acceptToken |=  int(token) == process.id
                        acceptFilter &= acceptToken
                    
                    if acceptFilter:
                        accept = True
                        break
                
                if accept:
                    
                    if filter_bullet is not None:
                        if filter_bullet == 'e' and isinstance(process.reactants.orderedContent()[0].value, zcross.Electron):
                            pass
                        elif filter_exact and filter_bullet != 'e' and isinstance(process.reactants.orderedContent()[0].value, zcross.Molecule) and filter_target == str(process.reactants.orderedContent()[0].value):
                           pass
                        else:
                            continue
                            
                    if filter_target is not None:
                        if filter_target == 'e' and isinstance(process.reactants[1].value, zcross.Electron):
                            pass
                        elif not filter_exact and filter_target != 'e' and isinstance(process.reactants.orderedContent()[1].value, zcross.Molecule) and filter_target == process.reactants.orderedContent()[1].value.orderedContent()[0].value:
                            pass
                        elif     filter_exact and filter_target != 'e' and isinstance(process.reactants.orderedContent()[1].value, zcross.Molecule) and filter_target == str(process.reactants.orderedContent()[1].value):
                           pass
                        else:
                            continue
                            
                    if filter_collission is not None:
                        if filter_collission in ('total', 'elastic', 'inelastic', 'superelastic'):
                            if process.collisionType == filter_collission:
                                pass
                            else:
                                continue
                        elif filter_collission in ('excitation'):
                            if process.collisionType == 'inelastic' and process.inelasticType in ('excitation_ele', 'excitation_vib', 'excitation_rot'):
                                pass
                            else:
                                continue
                        elif filter_collission in ('excitation_ele', 'excitation_vib', 'excitation_rot', 'ionization', 'attachment', 'neutral'):
                            if process.collisionType == 'inelastic' and process.inelasticType == filter_collission:
                                pass
                            else:
                                continue
                        
                    
                    if process.data_x.units.lower() == 'hartee':
                        process_unit_energy   = ureg('Eh')
                    else:
                        process_unit_energy = ureg(process.data_x.units)
                    
                    if process.data_y.units == 'm2':
                        process_unit_area   = ureg('m^2')  
                    elif process.data_y.lower() == 'bohr2':
                        process_unit_area   = ureg('(hbar / (1./137. * m_e * c))^2')  # TODO: Replace with ureg('a0^2') when pint 0.10 will be released on pypi
                    else:
                        process_unit_area   = ureg(process.data_y.units)
                    
                    areas    = [ area   * process_unit_area   for area   in process.data_y.value() ]
                    
                    key = '{}/{}/{}'.format(database.id, group.id, process.id)
                    if not args.velocity:
                        energies = [ energy * process_unit_energy for energy in process.data_x.value() ]
                        processes[key] = (energies, areas)
                        
                    else:
                        bullet_mass = calculate_mass(process.reactants[0])
                        velocities = [ (2. * energy * process_unit_energy / bullet_mass) ** 0.5 for energy in process.energy ]
                        processes[key] = (velocities, areas)
                    
                    if args.legend == 'ids':
                        labels[key] = key 
                    elif args.legend == 'types':
                        labels[key] = process.get_simple_type()
                    else:
                        labels[key] = None
                    
    fig, ax1 = plt.subplots(dpi=150)

    #fig.set_figheight(15)
    #fig.set_figwidth(30)
    fig.set_figwidth(2 * fig.get_figheight())
    plt.title('Cross-sections tables' if args.title is None else args.title)
    
    
    ax1.set_axisbelow(True)
    ax1.grid(True, which='both', axis='both', color='#e0e0e0', linestyle='-', linewidth=1)
    ax1.minorticks_on()

    if not args.velocity:
        ax1.set_xlabel('Energy [{.units:~P}]'.format(unit_energy))
    else:
        ax1.set_xlabel('Velocity [{.units:~P}]'.format(unit_velocity))

    ax1.set_ylabel('Cross Section [{.units:~P}]'.format(unit_area))

    ax1.set_xscale('log')
    ax1.set_yscale('log')
        
        
    #mypalette=Spectral11[0:len(processes)]

    i = 0

    def get_ids(x):
         ids = x[0].split('/')
         ids[2] = int(ids[2])
         return ids


    ax1.set_prop_cycle(cycler(color=plt.get_cmap('tab20b').colors))


    for key, value in sorted(processes.items(), key=get_ids):
        
        if not args.velocity:
            x = [v.to(unit_energy).magnitude   for v in value[0]]
        else:
            x = [v.to(unit_velocity).magnitude for v in value[0]]
            
        y = [v.to(unit_area).magnitude  if v != 0. else None for v in value[1]]
        
    
        #x, y = zip(*[v for v in zip(x,y) if v[1] != 0])
        ax1.plot(x, y, label=labels[key] ) #, markersize=4., marker='.'
        #ax1.scatter(x, y, label=label, s = 10)
           
        
        i+=1
        
    if args.legend != 'none':
        ax1.legend( ncol=len(processes)//25+1, fontsize='xx-small', bbox_to_anchor=(1.02, 1), loc='upper left')  
    
    fig.tight_layout()

    if args.output is None: 
        plt.show()
    else:  
        print("Saving {}".format(args.output))
        plt.savefig(args.output)


if __name__ == "__main__":
    main()




# zcross/raw/binding.py
# -*- coding: utf-8 -*-
# PyXB bindings for NM:626ec537dfcf4e27e1a94ab21ad8ef1c6da8dfdf
# Generated 2021-09-05 16:09:07.053815 by PyXB version 1.2.6 using Python 3.7.3.final.0
# Namespace https://zcross.net

from __future__ import unicode_literals
import pyxb
import pyxb.binding
import pyxb.binding.saxer
import io
import pyxb.utils.utility
import pyxb.utils.domutils
import sys
import pyxb.utils.six as _six
# Unique identifier for bindings created at the same time
_GenerationUID = pyxb.utils.utility.UniqueIdentifier('urn:uuid:73865c2c-0e4a-11ec-8928-50eb711e81f1')

# Version of PyXB used to generate the bindings
_PyXBVersion = '1.2.6'
# Generated bindings are not compatible across PyXB versions
if pyxb.__version__ != _PyXBVersion:
    raise pyxb.PyXBVersionError(_PyXBVersion)

# A holder for module-level binding classes so we can access them from
# inside class definitions where property names may conflict.
_module_typeBindings = pyxb.utils.utility.Object()

# Import bindings for namespaces imported into schema
import pyxb.binding.datatypes

# NOTE: All namespace declarations are reserved within the binding
Namespace = pyxb.namespace.NamespaceForURI('https://zcross.net', create_if_missing=True)
Namespace.configureCategories(['typeBinding', 'elementBinding'])

def CreateFromDocument (xml_text, default_namespace=None, location_base=None):
    """Parse the given XML and use the document element to create a
    Python instance.

    @param xml_text An XML document.  This should be data (Python 2
    str or Python 3 bytes), or a text (Python 2 unicode or Python 3
    str) in the L{pyxb._InputEncoding} encoding.

    @keyword default_namespace The L{pyxb.Namespace} instance to use as the
    default namespace where there is no default namespace in scope.
    If unspecified or C{None}, the namespace of the module containing
    this function will be used.

    @keyword location_base: An object to be recorded as the base of all
    L{pyxb.utils.utility.Location} instances associated with events and
    objects handled by the parser.  You might pass the URI from which
    the document was obtained.
    """

    if pyxb.XMLStyle_saxer != pyxb._XMLStyle:
        dom = pyxb.utils.domutils.StringToDOM(xml_text)
        return CreateFromDOM(dom.documentElement, default_namespace=default_namespace)
    if default_namespace is None:
        default_namespace = Namespace.fallbackNamespace()
    saxer = pyxb.binding.saxer.make_parser(fallback_namespace=default_namespace, location_base=location_base)
    handler = saxer.getContentHandler()
    xmld = xml_text
    if isinstance(xmld, _six.text_type):
        xmld = xmld.encode(pyxb._InputEncoding)
    saxer.parse(io.BytesIO(xmld))
    instance = handler.rootObject()
    return instance

def CreateFromDOM (node, default_namespace=None):
    """Create a Python instance from the given DOM node.
    The node tag must correspond to an element declaration in this module.

    @deprecated: Forcing use of DOM interface is unnecessary; use L{CreateFromDocument}."""
    if default_namespace is None:
        default_namespace = Namespace.fallbackNamespace()
    return pyxb.binding.basis.element.AnyCreateFromDOM(node, default_namespace)


# List simple type: {https://zcross.net}ValueList
# superclasses pyxb.binding.datatypes.anySimpleType
class ValueList (pyxb.binding.basis.STD_list):

    """Simple type that is a list of pyxb.binding.datatypes.double."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'ValueList')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 156, 2)
    _Documentation = None

    _ItemType = pyxb.binding.datatypes.double
ValueList._InitializeFacetMap()
Namespace.addCategoryObject('typeBinding', 'ValueList', ValueList)
_module_typeBindings.ValueList = ValueList

# Atomic simple type: {https://zcross.net}TableType
class TableType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'TableType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 159, 2)
    _Documentation = None
TableType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=TableType, enum_prefix=None)
TableType.integral = TableType._CF_enumeration.addEnumeration(unicode_value='integral', tag='integral')
TableType.differential = TableType._CF_enumeration.addEnumeration(unicode_value='differential', tag='differential')
TableType._InitializeFacetMap(TableType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'TableType', TableType)
_module_typeBindings.TableType = TableType

# Atomic simple type: {https://zcross.net}CollisionType
class CollisionType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'CollisionType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 165, 2)
    _Documentation = None
CollisionType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=CollisionType, enum_prefix=None)
CollisionType.total = CollisionType._CF_enumeration.addEnumeration(unicode_value='total', tag='total')
CollisionType.elastic = CollisionType._CF_enumeration.addEnumeration(unicode_value='elastic', tag='elastic')
CollisionType.inelastic = CollisionType._CF_enumeration.addEnumeration(unicode_value='inelastic', tag='inelastic')
CollisionType.superelastic = CollisionType._CF_enumeration.addEnumeration(unicode_value='superelastic', tag='superelastic')
CollisionType._InitializeFacetMap(CollisionType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'CollisionType', CollisionType)
_module_typeBindings.CollisionType = CollisionType

# Atomic simple type: {https://zcross.net}InelasticType
class InelasticType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'InelasticType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 173, 2)
    _Documentation = None
InelasticType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=InelasticType, enum_prefix=None)
InelasticType.excitation_ele = InelasticType._CF_enumeration.addEnumeration(unicode_value='excitation_ele', tag='excitation_ele')
InelasticType.excitation_vib = InelasticType._CF_enumeration.addEnumeration(unicode_value='excitation_vib', tag='excitation_vib')
InelasticType.excitation_rot = InelasticType._CF_enumeration.addEnumeration(unicode_value='excitation_rot', tag='excitation_rot')
InelasticType.ionization = InelasticType._CF_enumeration.addEnumeration(unicode_value='ionization', tag='ionization')
InelasticType.attachment = InelasticType._CF_enumeration.addEnumeration(unicode_value='attachment', tag='attachment')
InelasticType.neutral = InelasticType._CF_enumeration.addEnumeration(unicode_value='neutral', tag='neutral')
InelasticType._InitializeFacetMap(InelasticType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'InelasticType', InelasticType)
_module_typeBindings.InelasticType = InelasticType

# Atomic simple type: {https://zcross.net}DirectionType
class DirectionType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'DirectionType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 183, 2)
    _Documentation = None
DirectionType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=DirectionType, enum_prefix=None)
DirectionType.isotropic = DirectionType._CF_enumeration.addEnumeration(unicode_value='isotropic', tag='isotropic')
DirectionType.anisotropic = DirectionType._CF_enumeration.addEnumeration(unicode_value='anisotropic', tag='anisotropic')
DirectionType.backscattering = DirectionType._CF_enumeration.addEnumeration(unicode_value='backscattering', tag='backscattering')
DirectionType._InitializeFacetMap(DirectionType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'DirectionType', DirectionType)
_module_typeBindings.DirectionType = DirectionType

# Atomic simple type: {https://zcross.net}SourceType
class SourceType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'SourceType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 190, 2)
    _Documentation = None
SourceType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=SourceType, enum_prefix=None)
SourceType.experiment = SourceType._CF_enumeration.addEnumeration(unicode_value='experiment', tag='experiment')
SourceType.theory = SourceType._CF_enumeration.addEnumeration(unicode_value='theory', tag='theory')
SourceType.mixed = SourceType._CF_enumeration.addEnumeration(unicode_value='mixed', tag='mixed')
SourceType._InitializeFacetMap(SourceType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'SourceType', SourceType)
_module_typeBindings.SourceType = SourceType

# Atomic simple type: {https://zcross.net}DataType
class DataType (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'DataType')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 197, 2)
    _Documentation = None
DataType._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=DataType, enum_prefix=None)
DataType.angle = DataType._CF_enumeration.addEnumeration(unicode_value='angle', tag='angle')
DataType.energy = DataType._CF_enumeration.addEnumeration(unicode_value='energy', tag='energy')
DataType.cross_section = DataType._CF_enumeration.addEnumeration(unicode_value='cross_section', tag='cross_section')
DataType._InitializeFacetMap(DataType._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'DataType', DataType)
_module_typeBindings.DataType = DataType

# Union simple type: {https://zcross.net}ParameterValue
# superclasses pyxb.binding.datatypes.anySimpleType
class ParameterValue (pyxb.binding.basis.STD_union):

    """Simple type that is a union of pyxb.binding.datatypes.boolean, pyxb.binding.datatypes.integer, pyxb.binding.datatypes.decimal, pyxb.binding.datatypes.double."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'ParameterValue')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 204, 2)
    _Documentation = None

    _MemberTypes = ( pyxb.binding.datatypes.boolean, pyxb.binding.datatypes.integer, pyxb.binding.datatypes.decimal, pyxb.binding.datatypes.double, )
ParameterValue._CF_pattern = pyxb.binding.facets.CF_pattern()
ParameterValue._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=ParameterValue)
ParameterValue._InitializeFacetMap(ParameterValue._CF_pattern,
   ParameterValue._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'ParameterValue', ParameterValue)
_module_typeBindings.ParameterValue = ParameterValue

# Complex type {https://zcross.net}ZCross with content type ELEMENT_ONLY
class ZCross (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}ZCross with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'ZCross')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 4, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}database uses Python identifier database
    __database = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'database'), 'database', '__httpszcross_net_ZCross_httpszcross_netdatabase', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 11, 2), )

    
    database = property(__database.value, __database.set, None, None)

    
    # Attribute created uses Python identifier created
    __created = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'created'), 'created', '__httpszcross_net_ZCross_created', pyxb.binding.datatypes.string, required=True)
    __created._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 8, 4)
    __created._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 8, 4)
    
    created = property(__created.value, __created.set, None, None)

    
    # Attribute version uses Python identifier version
    __version = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'version'), 'version', '__httpszcross_net_ZCross_version', pyxb.binding.datatypes.string, required=True)
    __version._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 9, 4)
    __version._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 9, 4)
    
    version = property(__version.value, __version.set, None, None)

    _ElementMap.update({
        __database.name() : __database
    })
    _AttributeMap.update({
        __created.name() : __created,
        __version.name() : __version
    })
_module_typeBindings.ZCross = ZCross
Namespace.addCategoryObject('typeBinding', 'ZCross', ZCross)


# Complex type {https://zcross.net}Database with content type ELEMENT_ONLY
class Database (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Database with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Database')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 12, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}name uses Python identifier name
    __name = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'name'), 'name', '__httpszcross_net_Database_httpszcross_netname', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 14, 6), )

    
    name = property(__name.value, __name.set, None, None)

    
    # Element {https://zcross.net}description uses Python identifier description
    __description = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'description'), 'description', '__httpszcross_net_Database_httpszcross_netdescription', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 15, 6), )

    
    description = property(__description.value, __description.set, None, None)

    
    # Element {https://zcross.net}contact uses Python identifier contact
    __contact = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'contact'), 'contact', '__httpszcross_net_Database_httpszcross_netcontact', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 16, 6), )

    
    contact = property(__contact.value, __contact.set, None, None)

    
    # Element {https://zcross.net}url uses Python identifier url
    __url = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'url'), 'url', '__httpszcross_net_Database_httpszcross_neturl', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 17, 6), )

    
    url = property(__url.value, __url.set, None, None)

    
    # Element {https://zcross.net}references uses Python identifier references
    __references = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'references'), 'references', '__httpszcross_net_Database_httpszcross_netreferences', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 25, 2), )

    
    references = property(__references.value, __references.set, None, None)

    
    # Element {https://zcross.net}groups uses Python identifier groups
    __groups = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'groups'), 'groups', '__httpszcross_net_Database_httpszcross_netgroups', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 39, 2), )

    
    groups = property(__groups.value, __groups.set, None, None)

    
    # Attribute id uses Python identifier id
    __id = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'id'), 'id', '__httpszcross_net_Database_id', pyxb.binding.datatypes.string, required=True)
    __id._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 21, 4)
    __id._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 21, 4)
    
    id = property(__id.value, __id.set, None, None)

    
    # Attribute version uses Python identifier version
    __version = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'version'), 'version', '__httpszcross_net_Database_version', pyxb.binding.datatypes.string)
    __version._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 22, 4)
    __version._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 22, 4)
    
    version = property(__version.value, __version.set, None, None)

    
    # Attribute refs uses Python identifier refs
    __refs = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'refs'), 'refs', '__httpszcross_net_Database_refs', pyxb.binding.datatypes.string)
    __refs._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 23, 4)
    __refs._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 23, 4)
    
    refs = property(__refs.value, __refs.set, None, None)

    _ElementMap.update({
        __name.name() : __name,
        __description.name() : __description,
        __contact.name() : __contact,
        __url.name() : __url,
        __references.name() : __references,
        __groups.name() : __groups
    })
    _AttributeMap.update({
        __id.name() : __id,
        __version.name() : __version,
        __refs.name() : __refs
    })
_module_typeBindings.Database = Database
Namespace.addCategoryObject('typeBinding', 'Database', Database)


# Complex type {https://zcross.net}References with content type MIXED
class References (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}References with content type MIXED"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_MIXED
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'References')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 26, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}reference uses Python identifier reference
    __reference = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'reference'), 'reference', '__httpszcross_net_References_httpszcross_netreference', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 31, 2), )

    
    reference = property(__reference.value, __reference.set, None, None)

    _ElementMap.update({
        __reference.name() : __reference
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.References = References
Namespace.addCategoryObject('typeBinding', 'References', References)


# Complex type {https://zcross.net}Reference with content type SIMPLE
class Reference (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Reference with content type SIMPLE"""
    _TypeDefinition = pyxb.binding.datatypes.string
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_SIMPLE
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Reference')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 32, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.string
    
    # Attribute id uses Python identifier id
    __id = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'id'), 'id', '__httpszcross_net_Reference_id', pyxb.binding.datatypes.int, required=True)
    __id._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 35, 8)
    __id._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 35, 8)
    
    id = property(__id.value, __id.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __id.name() : __id
    })
_module_typeBindings.Reference = Reference
Namespace.addCategoryObject('typeBinding', 'Reference', Reference)


# Complex type {https://zcross.net}Groups with content type ELEMENT_ONLY
class Groups (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Groups with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Groups')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 40, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}group uses Python identifier group
    __group = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'group'), 'group', '__httpszcross_net_Groups_httpszcross_netgroup', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 45, 2), )

    
    group = property(__group.value, __group.set, None, None)

    _ElementMap.update({
        __group.name() : __group
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Groups = Groups
Namespace.addCategoryObject('typeBinding', 'Groups', Groups)


# Complex type {https://zcross.net}Processes with content type ELEMENT_ONLY
class Processes (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Processes with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Processes')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 58, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}process uses Python identifier process
    __process = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'process'), 'process', '__httpszcross_net_Processes_httpszcross_netprocess', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 63, 2), )

    
    process = property(__process.value, __process.set, None, None)

    _ElementMap.update({
        __process.name() : __process
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Processes = Processes
Namespace.addCategoryObject('typeBinding', 'Processes', Processes)


# Complex type {https://zcross.net}Reactants with content type ELEMENT_ONLY
class Reactants (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Reactants with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Reactants')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 87, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}electron uses Python identifier electron
    __electron = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'electron'), 'electron', '__httpszcross_net_Reactants_httpszcross_netelectron', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 106, 2), )

    
    electron = property(__electron.value, __electron.set, None, None)

    
    # Element {https://zcross.net}molecule uses Python identifier molecule
    __molecule = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'molecule'), 'molecule', '__httpszcross_net_Reactants_httpszcross_netmolecule', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 108, 2), )

    
    molecule = property(__molecule.value, __molecule.set, None, None)

    _ElementMap.update({
        __electron.name() : __electron,
        __molecule.name() : __molecule
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Reactants = Reactants
Namespace.addCategoryObject('typeBinding', 'Reactants', Reactants)


# Complex type {https://zcross.net}Products with content type ELEMENT_ONLY
class Products (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Products with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Products')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 94, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}electron uses Python identifier electron
    __electron = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'electron'), 'electron', '__httpszcross_net_Products_httpszcross_netelectron', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 106, 2), )

    
    electron = property(__electron.value, __electron.set, None, None)

    
    # Element {https://zcross.net}molecule uses Python identifier molecule
    __molecule = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'molecule'), 'molecule', '__httpszcross_net_Products_httpszcross_netmolecule', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 108, 2), )

    
    molecule = property(__molecule.value, __molecule.set, None, None)

    
    # Attribute weight uses Python identifier weight
    __weight = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'weight'), 'weight', '__httpszcross_net_Products_weight', pyxb.binding.datatypes.double)
    __weight._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 99, 4)
    __weight._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 99, 4)
    
    weight = property(__weight.value, __weight.set, None, None)

    _ElementMap.update({
        __electron.name() : __electron,
        __molecule.name() : __molecule
    })
    _AttributeMap.update({
        __weight.name() : __weight
    })
_module_typeBindings.Products = Products
Namespace.addCategoryObject('typeBinding', 'Products', Products)


# Complex type {https://zcross.net}Reactant with content type MIXED
class Reactant (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Reactant with content type MIXED"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_MIXED
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Reactant')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 102, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Attribute state uses Python identifier state
    __state = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'state'), 'state', '__httpszcross_net_Reactant_state', pyxb.binding.datatypes.string)
    __state._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 103, 4)
    __state._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 103, 4)
    
    state = property(__state.value, __state.set, None, None)

    
    # Attribute charge uses Python identifier charge
    __charge = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'charge'), 'charge', '__httpszcross_net_Reactant_charge', pyxb.binding.datatypes.integer)
    __charge._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 104, 4)
    __charge._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 104, 4)
    
    charge = property(__charge.value, __charge.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __state.name() : __state,
        __charge.name() : __charge
    })
_module_typeBindings.Reactant = Reactant
Namespace.addCategoryObject('typeBinding', 'Reactant', Reactant)


# Complex type {https://zcross.net}Electron with content type EMPTY
class Electron (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Electron with content type EMPTY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_EMPTY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Electron')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 107, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    _ElementMap.update({
        
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Electron = Electron
Namespace.addCategoryObject('typeBinding', 'Electron', Electron)


# Complex type {https://zcross.net}Molecule with content type MIXED
class Molecule (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Molecule with content type MIXED"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_MIXED
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Molecule')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 109, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Attribute state uses Python identifier state
    __state = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'state'), 'state', '__httpszcross_net_Molecule_state', pyxb.binding.datatypes.string)
    __state._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 110, 4)
    __state._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 110, 4)
    
    state = property(__state.value, __state.set, None, None)

    
    # Attribute charge uses Python identifier charge
    __charge = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'charge'), 'charge', '__httpszcross_net_Molecule_charge', pyxb.binding.datatypes.integer)
    __charge._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 111, 4)
    __charge._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 111, 4)
    
    charge = property(__charge.value, __charge.set, None, None)

    
    # Attribute isomer uses Python identifier isomer
    __isomer = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'isomer'), 'isomer', '__httpszcross_net_Molecule_isomer', pyxb.binding.datatypes.string)
    __isomer._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 112, 4)
    __isomer._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 112, 4)
    
    isomer = property(__isomer.value, __isomer.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __state.name() : __state,
        __charge.name() : __charge,
        __isomer.name() : __isomer
    })
_module_typeBindings.Molecule = Molecule
Namespace.addCategoryObject('typeBinding', 'Molecule', Molecule)


# Complex type {https://zcross.net}Parameters with content type ELEMENT_ONLY
class Parameters (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Parameters with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Parameters')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 115, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}parameter uses Python identifier parameter
    __parameter = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'parameter'), 'parameter', '__httpszcross_net_Parameters_httpszcross_netparameter', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 147, 2), )

    
    parameter = property(__parameter.value, __parameter.set, None, None)

    _ElementMap.update({
        __parameter.name() : __parameter
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Parameters = Parameters
Namespace.addCategoryObject('typeBinding', 'Parameters', Parameters)


# Complex type {https://zcross.net}Group with content type ELEMENT_ONLY
class Group (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Group with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Group')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 46, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}description uses Python identifier description
    __description = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'description'), 'description', '__httpszcross_net_Group_httpszcross_netdescription', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 48, 6), )

    
    description = property(__description.value, __description.set, None, None)

    
    # Element {https://zcross.net}processes uses Python identifier processes
    __processes = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'processes'), 'processes', '__httpszcross_net_Group_httpszcross_netprocesses', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 57, 2), )

    
    processes = property(__processes.value, __processes.set, None, None)

    
    # Attribute id uses Python identifier id
    __id = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'id'), 'id', '__httpszcross_net_Group_id', pyxb.binding.datatypes.string, required=True)
    __id._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 51, 4)
    __id._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 51, 4)
    
    id = property(__id.value, __id.set, None, None)

    
    # Attribute alias uses Python identifier alias
    __alias = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'alias'), 'alias', '__httpszcross_net_Group_alias', pyxb.binding.datatypes.string)
    __alias._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 52, 4)
    __alias._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 52, 4)
    
    alias = property(__alias.value, __alias.set, None, None)

    
    # Attribute sourceType uses Python identifier sourceType
    __sourceType = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'sourceType'), 'sourceType', '__httpszcross_net_Group_sourceType', _module_typeBindings.SourceType, required=True)
    __sourceType._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 53, 4)
    __sourceType._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 53, 4)
    
    sourceType = property(__sourceType.value, __sourceType.set, None, None)

    
    # Attribute sourceMethod uses Python identifier sourceMethod
    __sourceMethod = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'sourceMethod'), 'sourceMethod', '__httpszcross_net_Group_sourceMethod', pyxb.binding.datatypes.string)
    __sourceMethod._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 54, 4)
    __sourceMethod._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 54, 4)
    
    sourceMethod = property(__sourceMethod.value, __sourceMethod.set, None, None)

    
    # Attribute refs uses Python identifier refs
    __refs = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'refs'), 'refs', '__httpszcross_net_Group_refs', pyxb.binding.datatypes.string)
    __refs._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 55, 4)
    __refs._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 55, 4)
    
    refs = property(__refs.value, __refs.set, None, None)

    _ElementMap.update({
        __description.name() : __description,
        __processes.name() : __processes
    })
    _AttributeMap.update({
        __id.name() : __id,
        __alias.name() : __alias,
        __sourceType.name() : __sourceType,
        __sourceMethod.name() : __sourceMethod,
        __refs.name() : __refs
    })
_module_typeBindings.Group = Group
Namespace.addCategoryObject('typeBinding', 'Group', Group)


# Complex type {https://zcross.net}Process with content type ELEMENT_ONLY
class Process (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Process with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Process')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 64, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element {https://zcross.net}comment uses Python identifier comment
    __comment = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'comment'), 'comment', '__httpszcross_net_Process_httpszcross_netcomment', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 69, 6), )

    
    comment = property(__comment.value, __comment.set, None, None)

    
    # Element {https://zcross.net}updated uses Python identifier updated
    __updated = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'updated'), 'updated', '__httpszcross_net_Process_httpszcross_netupdated', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 70, 6), )

    
    updated = property(__updated.value, __updated.set, None, None)

    
    # Element {https://zcross.net}size uses Python identifier size
    __size = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'size'), 'size', '__httpszcross_net_Process_httpszcross_netsize', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 71, 6), )

    
    size = property(__size.value, __size.set, None, None)

    
    # Element {https://zcross.net}reactants uses Python identifier reactants
    __reactants = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'reactants'), 'reactants', '__httpszcross_net_Process_httpszcross_netreactants', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 86, 2), )

    
    reactants = property(__reactants.value, __reactants.set, None, None)

    
    # Element {https://zcross.net}products uses Python identifier products
    __products = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'products'), 'products', '__httpszcross_net_Process_httpszcross_netproducts', True, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 93, 2), )

    
    products = property(__products.value, __products.set, None, None)

    
    # Element {https://zcross.net}parameters uses Python identifier parameters
    __parameters = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'parameters'), 'parameters', '__httpszcross_net_Process_httpszcross_netparameters', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 114, 2), )

    
    parameters = property(__parameters.value, __parameters.set, None, None)

    
    # Element {https://zcross.net}data_x uses Python identifier data_x
    __data_x = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'data_x'), 'data_x', '__httpszcross_net_Process_httpszcross_netdata_x', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 120, 2), )

    
    data_x = property(__data_x.value, __data_x.set, None, None)

    
    # Element {https://zcross.net}data_y uses Python identifier data_y
    __data_y = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'data_y'), 'data_y', '__httpszcross_net_Process_httpszcross_netdata_y', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 129, 2), )

    
    data_y = property(__data_y.value, __data_y.set, None, None)

    
    # Element {https://zcross.net}data_z uses Python identifier data_z
    __data_z = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(Namespace, 'data_z'), 'data_z', '__httpszcross_net_Process_httpszcross_netdata_z', False, pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 138, 2), )

    
    data_z = property(__data_z.value, __data_z.set, None, None)

    
    # Attribute id uses Python identifier id
    __id = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'id'), 'id', '__httpszcross_net_Process_id', pyxb.binding.datatypes.int)
    __id._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 76, 4)
    __id._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 76, 4)
    
    id = property(__id.value, __id.set, None, None)

    
    # Attribute tableType uses Python identifier tableType
    __tableType = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'tableType'), 'tableType', '__httpszcross_net_Process_tableType', _module_typeBindings.TableType, required=True)
    __tableType._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 77, 4)
    __tableType._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 77, 4)
    
    tableType = property(__tableType.value, __tableType.set, None, None)

    
    # Attribute collisionType uses Python identifier collisionType
    __collisionType = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'collisionType'), 'collisionType', '__httpszcross_net_Process_collisionType', _module_typeBindings.CollisionType, required=True)
    __collisionType._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 78, 4)
    __collisionType._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 78, 4)
    
    collisionType = property(__collisionType.value, __collisionType.set, None, None)

    
    # Attribute inelasticType uses Python identifier inelasticType
    __inelasticType = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'inelasticType'), 'inelasticType', '__httpszcross_net_Process_inelasticType', _module_typeBindings.InelasticType)
    __inelasticType._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 79, 4)
    __inelasticType._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 79, 4)
    
    inelasticType = property(__inelasticType.value, __inelasticType.set, None, None)

    
    # Attribute dissociative uses Python identifier dissociative
    __dissociative = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'dissociative'), 'dissociative', '__httpszcross_net_Process_dissociative', pyxb.binding.datatypes.boolean)
    __dissociative._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 80, 4)
    __dissociative._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 80, 4)
    
    dissociative = property(__dissociative.value, __dissociative.set, None, None)

    
    # Attribute directionType uses Python identifier directionType
    __directionType = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'directionType'), 'directionType', '__httpszcross_net_Process_directionType', _module_typeBindings.DirectionType)
    __directionType._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 81, 4)
    __directionType._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 81, 4)
    
    directionType = property(__directionType.value, __directionType.set, None, None)

    
    # Attribute momentOrder uses Python identifier momentOrder
    __momentOrder = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'momentOrder'), 'momentOrder', '__httpszcross_net_Process_momentOrder', pyxb.binding.datatypes.integer, required=True)
    __momentOrder._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 82, 4)
    __momentOrder._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 82, 4)
    
    momentOrder = property(__momentOrder.value, __momentOrder.set, None, None)

    
    # Attribute refs uses Python identifier refs
    __refs = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'refs'), 'refs', '__httpszcross_net_Process_refs', pyxb.binding.datatypes.string)
    __refs._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 83, 4)
    __refs._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 83, 4)
    
    refs = property(__refs.value, __refs.set, None, None)

    
    # Attribute sourceHash uses Python identifier sourceHash
    __sourceHash = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'sourceHash'), 'sourceHash', '__httpszcross_net_Process_sourceHash', pyxb.binding.datatypes.string)
    __sourceHash._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 84, 4)
    __sourceHash._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 84, 4)
    
    sourceHash = property(__sourceHash.value, __sourceHash.set, None, None)

    _ElementMap.update({
        __comment.name() : __comment,
        __updated.name() : __updated,
        __size.name() : __size,
        __reactants.name() : __reactants,
        __products.name() : __products,
        __parameters.name() : __parameters,
        __data_x.name() : __data_x,
        __data_y.name() : __data_y,
        __data_z.name() : __data_z
    })
    _AttributeMap.update({
        __id.name() : __id,
        __tableType.name() : __tableType,
        __collisionType.name() : __collisionType,
        __inelasticType.name() : __inelasticType,
        __dissociative.name() : __dissociative,
        __directionType.name() : __directionType,
        __momentOrder.name() : __momentOrder,
        __refs.name() : __refs,
        __sourceHash.name() : __sourceHash
    })
_module_typeBindings.Process = Process
Namespace.addCategoryObject('typeBinding', 'Process', Process)


# Complex type {https://zcross.net}DataX with content type SIMPLE
class DataX (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}DataX with content type SIMPLE"""
    _TypeDefinition = ValueList
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_SIMPLE
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'DataX')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 121, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is ValueList
    
    # Attribute type uses Python identifier type
    __type = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'type'), 'type', '__httpszcross_net_DataX_type', _module_typeBindings.DataType, required=True)
    __type._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 124, 8)
    __type._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 124, 8)
    
    type = property(__type.value, __type.set, None, None)

    
    # Attribute units uses Python identifier units
    __units = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'units'), 'units', '__httpszcross_net_DataX_units', pyxb.binding.datatypes.string, required=True)
    __units._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 125, 8)
    __units._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 125, 8)
    
    units = property(__units.value, __units.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __type.name() : __type,
        __units.name() : __units
    })
_module_typeBindings.DataX = DataX
Namespace.addCategoryObject('typeBinding', 'DataX', DataX)


# Complex type {https://zcross.net}DataY with content type SIMPLE
class DataY (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}DataY with content type SIMPLE"""
    _TypeDefinition = ValueList
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_SIMPLE
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'DataY')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 130, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is ValueList
    
    # Attribute type uses Python identifier type
    __type = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'type'), 'type', '__httpszcross_net_DataY_type', _module_typeBindings.DataType, required=True)
    __type._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 133, 8)
    __type._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 133, 8)
    
    type = property(__type.value, __type.set, None, None)

    
    # Attribute units uses Python identifier units
    __units = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'units'), 'units', '__httpszcross_net_DataY_units', pyxb.binding.datatypes.string, required=True)
    __units._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 134, 8)
    __units._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 134, 8)
    
    units = property(__units.value, __units.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __type.name() : __type,
        __units.name() : __units
    })
_module_typeBindings.DataY = DataY
Namespace.addCategoryObject('typeBinding', 'DataY', DataY)


# Complex type {https://zcross.net}DataZ with content type SIMPLE
class DataZ (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}DataZ with content type SIMPLE"""
    _TypeDefinition = ValueList
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_SIMPLE
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'DataZ')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 139, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is ValueList
    
    # Attribute type uses Python identifier type
    __type = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'type'), 'type', '__httpszcross_net_DataZ_type', _module_typeBindings.DataType, required=True)
    __type._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 142, 8)
    __type._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 142, 8)
    
    type = property(__type.value, __type.set, None, None)

    
    # Attribute units uses Python identifier units
    __units = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'units'), 'units', '__httpszcross_net_DataZ_units', pyxb.binding.datatypes.string, required=True)
    __units._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 143, 8)
    __units._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 143, 8)
    
    units = property(__units.value, __units.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __type.name() : __type,
        __units.name() : __units
    })
_module_typeBindings.DataZ = DataZ
Namespace.addCategoryObject('typeBinding', 'DataZ', DataZ)


# Complex type {https://zcross.net}Parameter with content type SIMPLE
class Parameter (pyxb.binding.basis.complexTypeDefinition):
    """Complex type {https://zcross.net}Parameter with content type SIMPLE"""
    _TypeDefinition = ParameterValue
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_SIMPLE
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Parameter')
    _XSDLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 148, 2)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is ParameterValue
    
    # Attribute name uses Python identifier name
    __name = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'name'), 'name', '__httpszcross_net_Parameter_name', pyxb.binding.datatypes.string, required=True)
    __name._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 151, 8)
    __name._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 151, 8)
    
    name = property(__name.value, __name.set, None, None)

    
    # Attribute units uses Python identifier units
    __units = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'units'), 'units', '__httpszcross_net_Parameter_units', pyxb.binding.datatypes.string)
    __units._DeclarationLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 152, 8)
    __units._UseLocation = pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 152, 8)
    
    units = property(__units.value, __units.set, None, None)

    _ElementMap.update({
        
    })
    _AttributeMap.update({
        __name.name() : __name,
        __units.name() : __units
    })
_module_typeBindings.Parameter = Parameter
Namespace.addCategoryObject('typeBinding', 'Parameter', Parameter)


zcross = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'zcross'), ZCross, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 3, 2))
Namespace.addCategoryObject('elementBinding', zcross.name().localName(), zcross)

database = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'database'), Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 11, 2))
Namespace.addCategoryObject('elementBinding', database.name().localName(), database)

references = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'references'), References, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 25, 2))
Namespace.addCategoryObject('elementBinding', references.name().localName(), references)

reference = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'reference'), Reference, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 31, 2))
Namespace.addCategoryObject('elementBinding', reference.name().localName(), reference)

groups = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'groups'), Groups, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 39, 2))
Namespace.addCategoryObject('elementBinding', groups.name().localName(), groups)

processes = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'processes'), Processes, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 57, 2))
Namespace.addCategoryObject('elementBinding', processes.name().localName(), processes)

reactants = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'reactants'), Reactants, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 86, 2))
Namespace.addCategoryObject('elementBinding', reactants.name().localName(), reactants)

products = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'products'), Products, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 93, 2))
Namespace.addCategoryObject('elementBinding', products.name().localName(), products)

reactant = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'reactant'), Reactant, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 101, 2))
Namespace.addCategoryObject('elementBinding', reactant.name().localName(), reactant)

electron = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'electron'), Electron, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 106, 2))
Namespace.addCategoryObject('elementBinding', electron.name().localName(), electron)

molecule = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'molecule'), Molecule, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 108, 2))
Namespace.addCategoryObject('elementBinding', molecule.name().localName(), molecule)

parameters = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'parameters'), Parameters, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 114, 2))
Namespace.addCategoryObject('elementBinding', parameters.name().localName(), parameters)

group = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'group'), Group, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 45, 2))
Namespace.addCategoryObject('elementBinding', group.name().localName(), group)

process = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'process'), Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 63, 2))
Namespace.addCategoryObject('elementBinding', process.name().localName(), process)

data_x = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_x'), DataX, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 120, 2))
Namespace.addCategoryObject('elementBinding', data_x.name().localName(), data_x)

data_y = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_y'), DataY, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 129, 2))
Namespace.addCategoryObject('elementBinding', data_y.name().localName(), data_y)

data_z = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_z'), DataZ, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 138, 2))
Namespace.addCategoryObject('elementBinding', data_z.name().localName(), data_z)

parameter = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'parameter'), Parameter, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 147, 2))
Namespace.addCategoryObject('elementBinding', parameter.name().localName(), parameter)



ZCross._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'database'), Database, scope=ZCross, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 11, 2)))

def _BuildAutomaton ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton
    del _BuildAutomaton
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(ZCross._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'database')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 6, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
ZCross._Automaton = _BuildAutomaton()




Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'name'), pyxb.binding.datatypes.string, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 14, 6)))

Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'description'), pyxb.binding.datatypes.string, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 15, 6)))

Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'contact'), pyxb.binding.datatypes.string, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 16, 6)))

Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'url'), pyxb.binding.datatypes.string, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 17, 6)))

Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'references'), References, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 25, 2)))

Database._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'groups'), Groups, scope=Database, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 39, 2)))

def _BuildAutomaton_ ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_
    del _BuildAutomaton_
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 16, 6))
    counters.add(cc_0)
    cc_1 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 17, 6))
    counters.add(cc_1)
    states = []
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'name')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 14, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'description')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 15, 6))
    st_1 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'contact')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 16, 6))
    st_2 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_2)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'url')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 17, 6))
    st_3 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_3)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'references')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 18, 6))
    st_4 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_4)
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Database._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'groups')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 19, 6))
    st_5 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_5)
    transitions = []
    transitions.append(fac.Transition(st_1, [
         ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_2, [
         ]))
    transitions.append(fac.Transition(st_3, [
         ]))
    transitions.append(fac.Transition(st_4, [
         ]))
    st_1._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_2, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_0, False) ]))
    st_2._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_1, True) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_1, False) ]))
    st_3._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_5, [
         ]))
    st_4._set_transitionSet(transitions)
    transitions = []
    st_5._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Database._Automaton = _BuildAutomaton_()




References._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'reference'), Reference, scope=References, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 31, 2)))

def _BuildAutomaton_2 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_2
    del _BuildAutomaton_2
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=None, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 28, 6))
    counters.add(cc_0)
    states = []
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(References._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'reference')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 28, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, True, containing_state=None)
References._Automaton = _BuildAutomaton_2()




Groups._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'group'), Group, scope=Groups, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 45, 2)))

def _BuildAutomaton_3 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_3
    del _BuildAutomaton_3
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Groups._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'group')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 42, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
         ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Groups._Automaton = _BuildAutomaton_3()




Processes._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'process'), Process, scope=Processes, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 63, 2)))

def _BuildAutomaton_4 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_4
    del _BuildAutomaton_4
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Processes._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'process')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 60, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
         ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Processes._Automaton = _BuildAutomaton_4()




Reactants._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'electron'), Electron, scope=Reactants, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 106, 2)))

Reactants._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'molecule'), Molecule, scope=Reactants, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 108, 2)))

def _BuildAutomaton_5 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_5
    del _BuildAutomaton_5
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=2, max=2, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 88, 4))
    counters.add(cc_0)
    states = []
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(Reactants._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'electron')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 89, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(Reactants._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'molecule')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 90, 6))
    st_1 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_1._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Reactants._Automaton = _BuildAutomaton_5()




Products._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'electron'), Electron, scope=Products, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 106, 2)))

Products._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'molecule'), Molecule, scope=Products, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 108, 2)))

def _BuildAutomaton_6 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_6
    del _BuildAutomaton_6
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Products._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'electron')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 96, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Products._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'molecule')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 97, 6))
    st_1 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    transitions = []
    transitions.append(fac.Transition(st_0, [
         ]))
    transitions.append(fac.Transition(st_1, [
         ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_0, [
         ]))
    transitions.append(fac.Transition(st_1, [
         ]))
    st_1._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Products._Automaton = _BuildAutomaton_6()




def _BuildAutomaton_7 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_7
    del _BuildAutomaton_7
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    return fac.Automaton(states, counters, True, containing_state=None)
Reactant._Automaton = _BuildAutomaton_7()




def _BuildAutomaton_8 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_8
    del _BuildAutomaton_8
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    return fac.Automaton(states, counters, True, containing_state=None)
Molecule._Automaton = _BuildAutomaton_8()




Parameters._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'parameter'), Parameter, scope=Parameters, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 147, 2)))

def _BuildAutomaton_9 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_9
    del _BuildAutomaton_9
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=None, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 116, 4))
    counters.add(cc_0)
    states = []
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(Parameters._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'parameter')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 117, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, True, containing_state=None)
Parameters._Automaton = _BuildAutomaton_9()




Group._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'description'), pyxb.binding.datatypes.string, scope=Group, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 48, 6)))

Group._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'processes'), Processes, scope=Group, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 57, 2)))

def _BuildAutomaton_10 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_10
    del _BuildAutomaton_10
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 48, 6))
    counters.add(cc_0)
    states = []
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Group._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'description')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 48, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Group._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'processes')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 49, 6))
    st_1 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_0, False) ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    st_1._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Group._Automaton = _BuildAutomaton_10()




Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'comment'), pyxb.binding.datatypes.string, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 69, 6)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'updated'), pyxb.binding.datatypes.dateTime, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 70, 6)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'size'), pyxb.binding.datatypes.integer, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 71, 6)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'reactants'), Reactants, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 86, 2)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'products'), Products, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 93, 2)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'parameters'), Parameters, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 114, 2)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_x'), DataX, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 120, 2)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_y'), DataY, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 129, 2)))

Process._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'data_z'), DataZ, scope=Process, location=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 138, 2)))

def _BuildAutomaton_11 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_11
    del _BuildAutomaton_11
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=None, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 67, 6))
    counters.add(cc_0)
    cc_1 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 74, 6))
    counters.add(cc_1)
    states = []
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'reactants')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 66, 6))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'products')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 67, 6))
    st_1 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'parameters')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 68, 6))
    st_2 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_2)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'comment')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 69, 6))
    st_3 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_3)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'updated')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 70, 6))
    st_4 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_4)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'size')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 71, 6))
    st_5 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_5)
    final_update = None
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'data_x')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 72, 6))
    st_6 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_6)
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'data_y')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 73, 6))
    st_7 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_7)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_1, False))
    symbol = pyxb.binding.content.ElementUse(Process._UseForTag(pyxb.namespace.ExpandedName(Namespace, 'data_z')), pyxb.utils.utility.Location('/opt/zcross/share/zcross/zcross.xsd', 74, 6))
    st_8 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_8)
    transitions = []
    transitions.append(fac.Transition(st_1, [
         ]))
    transitions.append(fac.Transition(st_2, [
         ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_2, [
        fac.UpdateInstruction(cc_0, False) ]))
    st_1._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_3, [
         ]))
    st_2._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_4, [
         ]))
    st_3._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_5, [
         ]))
    st_4._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_6, [
         ]))
    st_5._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_7, [
         ]))
    st_6._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_8, [
         ]))
    st_7._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_1, True) ]))
    st_8._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Process._Automaton = _BuildAutomaton_11()




import setuptools

# To publish on pypi:
# 1) Change version and delete sdist and bdist_wheel
# 2) python3 setup.py sdist bdist_wheel
# 3) twine upload --repository pypi dist/*

with open("README.md", "r") as fh:
    long_description = fh.read()
    
setuptools.setup(
     name='zcross',  
     version='0.0.11',
     scripts=[] ,
     author="Michele Renda",
     author_email="michele.renda@cern.ch",
     description="An utility to read low-pressure gaseous cross sections data",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://gitlab.com/micrenda/zcross-python",
     packages=setuptools.find_packages(),
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
         "Operating System :: OS Independent",
     ],
     install_requires=[
          'matplotlib',
          'pint',
     ],
     python_requires='>=3.0',
     entry_points = {
        'console_scripts': ['zcross-xml=zcross.zcross_xml:main', 'zcross-plot=zcross.zcross_plot:main'],
    }
 )



import xml.etree.ElementTree as et

import re
import os
import inspect

from zcross.raw.binding import *
import zcross.raw.binding as raw_binding

import pyxb.utils.domutils
pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace("https://zcross.net")

re_reference   = re.compile(r'@(\w+){([\w\-]+),([\s\S]*)}', re.MULTILINE)
re_whitespace  = re.compile(r'\s+', re.MULTILINE)
re_database_id = re.compile(r'\<database\s+id\s*=\s*\"([A-Za-z0-9]+)\"', re.MULTILINE)

def load_by_xml(filename):
    with open(filename, 'r') as f:
        return raw_binding.CreateFromDocument(f.read())
    
def save_to_xml(document, filename):
    
    with open(filename, 'w') as f:
        f.write(document.toxml("utf-8", element_name='zcross').decode('utf-8'))

def _get_data_dirs():
    data_dirs = []
    
    if 'ZCROSS_DATA' in os.environ:
        for token in os.environ['ZCROSS_DATA'].split(':'):
            if token:
                if os.path.exists(token) and os.path.isdir(token):
                    data_dirs.append(token)
                else:
                    raise Exception('Unable to access the directory \'{}\' specified in path ZCROSS_DATA env.'.format(token))  
                    
    elif os.path.exists('/opt/zcross_data') and os.path.isdir('/opt/zcross_data'):
        data_dirs.append('/opt/zcross_data')
        
    if data_dirs is None:
        raise Exception('Unable to find ZCross database directory path: define ZCROSS_DATA env variable.')
      
    return data_dirs
    
    
def load_by_name(database):
  
    for directory in _get_data_dirs():
        for r, d, f in os.walk(directory):
            for fx in f:
                if fx.endswith('.xml'):
                    filename = os.path.join(r, fx)
                    with open(filename, 'r') as f:
                        header = f.read(1024) # reading the first caracters of the XML files whitout loading everything in memory
                        m = re_database_id.search(header)
                        if m and m.group(1).lower() == database.lower():
                            f.seek(0)
                            return raw_binding.CreateFromDocument(f.read())                    
    raise Exception('Unable to find a database with name \'{}\''.format(database))
                        
     
    
def load_all(verbose = False):
  
    contents = []
   
    for r, d, f in os.walk(_get_data_dir()):
        for fx in f:
            if fx.endswith('.xml'):
                filename = os.path.join(r, fx)

                with open(filename, 'r') as f:
                    if verbose:
                        print('Loading {}'.format(filename))
                    contents.append(raw_binding.CreateFromDocument(f.read()))
    return contents
                    

    
class ZCross(raw_binding.ZCross):
    pass
    
class DatabaseIterator:

    def __init__(self, database):
        self._child_iter = iter(database.groups.orderedContent())
        
    def __next__(self):
       return next(self._child_iter).value
    
class Database(raw_binding.Database):
    
    def __str__(self):
        return "Database({})".format(self.id)
        
    def __iter__(self):
        return DatabaseIterator(self)
        
    def __len__(self):
        return len(self.self.groups)
    

class GroupIterator:

    def __init__(self, group):
        self._child_iter = iter(group.processes.orderedContent())
        
    def __next__(self):
       return next(self._child_iter).value
    
class Group(raw_binding.Group):

    def __str__(self):
        return "Group({})".format(self.id)
        
    def __iter__(self):
        return GroupIterator(self)
    
    def __len__(self):
        return len(self.processes.orderedContent())
    
class Reference(raw_binding.Reference):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
         
        m = re_reference.match(self.value())
        
        if m:
            self.label = m.group(2)
            self.type = m.group(1)
            content = m.group(3)
            
            # Parsing content
            
            section = 0
            
            current_key = ''
            current_value = ''
                        
             
            level = 0
                     
            for c in content:

                if c == '=' and level == 0:
                    if section == 0:
                        section = 1
                        continue
                elif c == ',' and level == 0:
                    if section == 1:
                        section = 2
                        continue
                elif c == '{':
                    level += 1
                elif c == '}':
                    level -= 1
                    
                
                if section == 0:
                    current_key += c
                if section == 1:
                    current_value += c
                if section == 2:
                    key = current_key.strip()
                    value = current_value.strip()
                    
                    if value.startswith('{') and value.endswith('}'):
                        value = value.strip('{').strip('}')
                        
                    setattr(self, key, value)
                    
                    current_key   = ''
                    current_value = ''
                    level = 0
                    section = 0
       
    def bibtex(self):
        s = '@{}{{{},\n'.format(self.type, self.label)
        for k,v in vars(self).items():
            if k not in ['label','type','id']:
                s += '\t{} = {{{}}},\n'.format(k,v)
        s += '}'
        
        return s
        
      
class Process(raw_binding.Process):
    
    
    def get_reaction(self):
        s  = ' + '.join(str(r.value) for r in self.reactants.orderedContent())
        s += ' → '
        # This hurts :)
        s += ' or '.join(' + '.join(str(p.value) for p in ps.orderedContent()) for ps in self.products)
        
        
    def __str__(self):
        return 'Process({}: {})'.format(self.id, self.get_reaction())      
        
    def get_nano_type(self):
        if self.collisionType == 'elastic':
            if self.momentOrder == 0:
                return  'el'
            elif self.momentOrder == 1:
                return  'mt'
            elif self.momentOrder == 2:
                return  'vs'
            else:
                return  'mo {}'.format(self.moment_order)
        elif self.collisionType == 'inelastic':
            if   self.inelasticType == 'excitation_ele':
                return  'ele'
            elif self.inelasticType == 'excitation_vib':
                return  'vib'
            elif self.inelasticType == 'excitation_rot':
                return  'rot'
            elif self.inelasticType == 'ionization':
                return  'ion'
            elif self.inelasticType == 'attachment':
                return  'att'
            elif self.inelasticType == 'neutral':
                return  'neu'
            else:
                return  '???'
        else:
            return  '???'   
        
    def get_tiny_type(self):
        if self.collisionType == 'elastic':
            if self.momentOrder == 0:
                return  'ela el'
            elif self.momentOrder == 1:
                return  'ela mt'
            elif self.momentOrder == 2:
                return  'ela vs'
            else:
                return  'ela {}'.format(self.moment_order)
        elif self.collisionType == 'inelastic':
            if   self.inelasticType == 'excitation_ele':
                return  'exc ele'
            elif self.inelasticType == 'excitation_vib':
                return  'exc vib'
            elif self.inelasticType == 'excitation_rot':
                return  'exc rot'
            elif self.inelasticType == 'ionization':
                return  'ion'
            elif self.inelasticType == 'attachment':
                return  'att'
            elif self.inelasticType == 'neutral':
                return  'neu'
            else:
                return  '???'
        else:
            return  '???'
              
    def get_simple_type(self):
        if self.collisionType == 'elastic':
            if self.momentOrder == 0:
                return  'ela el'
            elif self.momentOrder == 1:
                return  'ela mt'
            elif self.momentOrder == 2:
                return  'ela vs'
            else:
                return  'ela {}'.format(self.moment_order)
        elif self.collisionType == 'inelastic':
            if   self.inelasticType == 'excitation_ele':
                return  'exc ele' + (' (' + self.products[0].molecule[0].state + ')') if len(self.products) and len(self.products[0].molecule) and self.products[0].molecule[0].state is not None else ''
            elif self.inelasticType == 'excitation_vib':
                return  'exc vib' + (' (' + self.products[0].molecule[0].state + ')') if len(self.products) and len(self.products[0].molecule) and self.products[0].molecule[0].state is not None else ''
            elif self.inelasticType == 'excitation_rot':
                return  'exc rot' + (' (' + self.products[0].molecule[0].state + ')') if len(self.products) and len(self.products[0].molecule) and self.products[0].molecule[0].state is not None else ''
            elif self.inelasticType == 'ionization':
                return  'ion' + (' ' + str(self.products[0].molecule[0])) if len(self.products) and len(self.products[0].molecule) else ''
            elif self.inelasticType == 'attachment':
                return  'att' + (' ' + str(self.products[0].molecule[0])) if len(self.products) and len(self.products[0].molecule) else ''
            elif self.inelasticType == 'neutral':
                return  'neu'
            else:
                return  '???'
        else:
            return  '???'
   
        
class Electron(raw_binding.Electron):

    def __str__(self):
        return 'e'
        
        
molecule_pat1 = re.compile('([0-9]+)')
molecule_pat2 = re.compile('\s*([A-Z][a-z]?)([0-9]*)\s*')

class Molecule(raw_binding.Molecule):
        
    def __str__(self):
        
        s = self.orderedContent()[0].value
        
        if self.charge != None:
            if abs(self.charge) > 1:
                s += (get_scripted_number(abs(self.charge), True, False) if abs(self.charge) >  1 else '') 
                
            if self.charge > 0:
                s += '⁺'
                
            if self.charge < 0:
                s += '⁻'
                
        if self.state != None:
            s += '(' + self.state + ')'
        
        return s
        
    def get_latex(self):
        
        s = self.orderedContent()[0].value
        
        if self.charge != None:
            if abs(self.charge) > 1:
                s += '^{' + str(abs(self.charge)) + ('+' if self.charge > 0 else '-')  + '}'
                
            elif self.charge > 0:
                s += '^{' + '+' * self.charge + '}'
                
            elif self.charge < 0:
                 s += '^{' + '-' * (-self.charge) + '}'
                
        if self.state != None:
            s += '\text{(' + self.state + ')}'
        
        return s
        
        
    # def explode(self):
        
        # s = self.value
        # global_reciept = {}

        # for start,end,level in self.__find_parens():

            # local_reciept = {}
            
            # formula  = s[start+1:end]
            # multiply = 1 
            # m1 = molecule_pat1.match(s, end+1)
            # if m1:
                # multiply = int(m1.group(1))
                # end = m1.span(1)[1]
                
            # # Parsing   
            # m2 = molecule_pat2.match(formula)
            # while m2:
                # atom = m2.group(1)
                
                # qty  = 1
                # if m2.group(2):
                    # qty = int(m2.group(2))
                    
                # if atom not in local_reciept:
                    # local_reciept[atom] = 0
                # local_reciept[atom] += qty
                                
                # m2 = molecule_pat2.match(formula, m2.span(1)[1])
                
            
            # for atom, qty in local_reciept.items():
                
                # if atom not in global_reciept:
                    # global_reciept[atom] = 0
                
                # global_reciept[atom] += qty * multiply
                
            # # Cleaning
            # s = s[:start] + ' ' * (end-start+1) + s[end+1:]
                
        # return global_reciept
                 
            
            
        
    # def __find_parens(self):
        # results = []
        # pstack = []
        
        # results.append((-1, len(self.value), 0))

        # for i, c in enumerate(self.value):
            # if c == '(':
                # pstack.append(i)
            # elif c == ')':
                # if len(pstack) == 0:
                    # raise IndexError("No matching closing parens at: " + str(i))
                # results.append((pstack.pop(), i, len(pstack)+1))

        # if len(pstack) > 0:
            # raise IndexError("No matching opening parens at: " + str(pstack.pop()))

        # results.sort(key=lambda result: result[0], reverse=True)
        
        # return results
    
    
class Parameter(raw_binding.Parameter):
    def __str__(self):
        return self.name + ' = ' + str(self.value()) + (' ' + self.units if self.units is not None else '')
    
   
    
    
def get_scripted_number(number, superscript, forcePlus):

    if not isinstance(number, int):
        raise "Argument 'number' must be an int"

    digitsSuper = ['⁰','¹','²','³','⁴','⁵','⁶','⁷','⁸','⁹']
    digitsSub   = ['₀','₁','₂','₃','₄','₅','₆','₇','₈','₉']

    s= ''

    if number < 0:
        if superscript:
            s += '⁻';
        else:
            s += '₋';
    
    elif number > 0 and forcePlus:
        if superscript:
            s += '⁺'
        else:
            s += '₊'

    while number > 0:
        
        if superscript:
            s = digitsSuper[number % 10] + s
        else:
            s = digitsSub[number % 10] + s
            
        number //= 10
    

    return s




raw_binding.ZCross._SetSupersedingClass(ZCross)
raw_binding.Database._SetSupersedingClass(Database)
raw_binding.Group._SetSupersedingClass(Group)
raw_binding.Electron._SetSupersedingClass(Electron)
raw_binding.Molecule._SetSupersedingClass(Molecule)
raw_binding.Parameter._SetSupersedingClass(Parameter)
raw_binding.Process._SetSupersedingClass(Process)
raw_binding.Reference._SetSupersedingClass(Reference)





# Utility function to identify classes of interest
def _isSupersedable (cls):
    return inspect.isclass(cls) and issubclass(cls, pyxb.binding.basis._DynamicCreate_mixin)

def _injectClasses ():
    import sys
    import pyxb.binding.basis

    # All PyXB complex type definitions in the original module
    raw_classes = set([_o for (_, _o) in inspect.getmembers(raw_binding) if _isSupersedable(_o)])
    #print 'Original classes: %s' % (raw_classes,)

    # PyXB complex type definitions in this module that did not come
    # from the original import *.
    this_module = sys.modules[__name__]
    this_classes = set([_o for (_, _o) in inspect.getmembers(this_module) if _isSupersedable(_o) and _o not in raw_classes])
    this_classes_tuple = tuple(this_classes)
    #print 'This classes: %s' % (this_classes,)

    # Raw classes superseded by something in this module
    superseded_classes = set([ _o for _o in raw_classes if _o._SupersedingClass() in this_classes ])
    superseded_classes_tuple = tuple(superseded_classes)
    #print 'Superseded classes: %s' % (superseded_classes,)

    # Raw classes that are subclasses of something superseded by this
    # module, but that are not themselves superseded by this module
    need_supersedure_classes = set([_o for _o in raw_classes if issubclass(_o, superseded_classes_tuple) and _o not in superseded_classes])
    #print 'Need supersedure classes: %s' % (need_supersedure_classes,)

    # Add local definitions to supersede classes all of whose
    # ancestors have been superseded as necessary.
    while need_supersedure_classes:
        did_replacement = False
        new_need_supersedure_classes = set()
        for o in need_supersedure_classes:
            candidate = True
            # Build the new sequence of base classes while we check them.
            new_mro = []
            for super_o in o.__mro__:
                if super_o == o:
                    # Put the superseded class in its original position (probably first)
                    new_mro.append(o)
                    continue
                if super_o in need_supersedure_classes:
                    # Subclass of a class we haven't gotten to yet; put it off
                    candidate = False
                    break
                # Append the replacement or the original, as needed
                if super_o in superseded_classes:
                    new_mro.append(super_o._SupersedingClass())
                else:
                    new_mro.append(super_o)
            if not candidate:
                new_need_supersedure_classes.add(o)
                continue
            # Create a new class that subclasses the replacements
            name = o.__name__
            new_o = type(name, tuple(new_mro), o.__dict__.copy())
            # Install it in the module
            setattr(this_module, name, new_o)
            # Tell PyXB to use it as the superseding class
            o._SetSupersedingClass(new_o)
            # Record it so future passes will find it
            superseded_classes.add(o)
        assert need_supersedure_classes != new_need_supersedure_classes
        need_supersedure_classes = new_need_supersedure_classes

_injectClasses()

